# -*- coding: utf-8 -*-
# pylint: disable=expression-not-assigned,line-too-long
"""Working hours (Danish arbejdstimer) or not? API."""
import datetime as dti
import json
import os
import pathlib
import sys
from typing import List, Optional, Tuple, Union

DEBUG_VAR = 'ARBEJDSTIMER_DEBUG'
DEBUG = os.getenv(DEBUG_VAR)

ENCODING = 'utf-8'
ENCODING_ERRORS_POLICY = 'ignore'


DEFAULT_CONFIG_NAME = '.arbejdstimer.json'


def weekday() -> int:
    """Return current weekday."""
    return dti.date.today().isoweekday()


def no_weekend(day_number: int) -> bool:
    """Return if day number is weekend."""
    return day_number < 6


def verify_request(argv: Optional[List[str]]) -> Tuple[int, str, List[str]]:
    """Fail with grace."""
    if not argv or len(argv) != 2:
        return 2, 'received wrong number of arguments', ['']

    command, config = argv

    if command not in ('now'):
        return 2, 'received unknown command', ['']

    if not config:
        return 2, 'configuration missing', ['']

    config_path = pathlib.Path(str(config))
    if not config_path.is_file():
        return 1, f'config ({config_path}) is no file', ['']
    if not ''.join(config_path.suffixes).lower().endswith('.json'):
        return 1, 'config has not .json extension', ['']

    return 0, '', argv


def main(argv: Union[List[str], None] = None) -> int:
    """Drive the lookup."""
    error, message, strings = verify_request(argv)
    if error:
        print(message, file=sys.stderr)
        return error

    command, config = strings

    with open(config, 'rt', encoding=ENCODING) as handle:
        configuration = json.load(handle)

    print(f'configuration is ({configuration})')
    week_day = weekday()
    work_day = no_weekend(week_day)
    if work_day:
        print(f'Today ({dti.date.today()}) is not a weekend')
    else:
        print('Today is weekend.')
        return 1

    hour = dti.datetime.now().hour
    if 6 < hour < 17:
        print(f'At this hour ({hour}) is work time')
    else:
        print(f'No worktime at hour({hour}).')
        return 1


    return 0



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# pylint: disable=line-too-long
"""Commandline API gateway for arbejdstimer."""
import pathlib
import sys
from typing import List, Union

import typer

import arbejdstimer
import arbejdstimer.arbejdstimer as at

APP_NAME = 'Working hours (Danish arbejdstimer) or not?'
APP_ALIAS = 'arbejdstimer'
app = typer.Typer(
    add_completion=False,
    context_settings={'help_option_names': ['-h', '--help']},
    no_args_is_help=True,
)


@app.callback(invoke_without_command=True)
def callback(
    version: bool = typer.Option(
        False,
        '-V',
        '--version',
        help='Display the arbejdstimer version and exit',
        is_eager=True,
    )
) -> None:
    """
    Working hours (Danish arbejdstimer) or not?

    Given a configuration file detect if today is a work day and
    if at the time of request is a working hour.
    """
    if version:
        typer.echo(f'{APP_NAME} version {arbejdstimer.__version__}')
        raise typer.Exit()


@app.command('now')
def now(
    conf: str = typer.Option(
        '',
        '-c',
        '--config',
        help='Path to config file (default is $HOME/.arbejdstimer.json)',
        metavar='<configpath>',
    ),
) -> int:
    """
    Answer the question if now is a working hour.
    """
    command = 'now'
    config = conf if conf else pathlib.Path.home() / at.DEFAULT_CONFIG_NAME
    action = [command, str(config)]
    return sys.exit(at.main(action))


@app.command('version')
def app_version() -> None:
    """
    Display the arbejdstimer version and exit
    """
    callback(True)


# pylint: disable=expression-not-assigned
# @app.command()
def main(argv: Union[List[str], None] = None) -> int:
    """Delegate processing to functional module."""
    argv = sys.argv[1:] if argv is None else argv
    return at.main(argv)



# -*- coding: utf-8 -*-
# pylint: disable=line-too-long,missing-docstring,reimported,unused-import,unused-variable
import pathlib

import arbejdstimer.arbejdstimer as at


def test_ko_main():
    inp = str(pathlib.Path('tests', 'fixtures', 'basic', 'minimal-config.json'))
    assert at.main(['now', inp]) == 0


def test_ko_verify_request_too_few():
    assert at.verify_request([1]) == (2, 'received wrong number of arguments', [''])


def test_ko_verify_request_unknown_command():
    assert at.verify_request(['unknown', 'does not matter']) == (2, 'received unknown command', [''])


def test_ko_verify_request_falsy_input():
    argv = ['now', '']
    assert at.verify_request(argv) == (2, 'configuration missing', [''])



# -*- coding: utf-8 -*-
# pylint: disable=line-too-long,missing-docstring,reimported,unused-import,unused-variable
import pathlib

import click
import pytest

import arbejdstimer.cli as cli


def test_main_legacy_ok(capsys):
    inp = str(pathlib.Path('tests', 'fixtures', 'basic', 'minimal-config.json'))
    assert cli.main(['now', inp]) == 0
    out, err = capsys.readouterr()
    assert 'configuration is' in out.lower()
    assert not err


def test_version_ok(capsys):
    with pytest.raises(click.exceptions.Exit) as exec_info:
        cli.app_version() == 0
        assert exec_info.value.code == 0
        out, err = capsys.readouterr()
        assert 'version' in out.lower()
        assert not err


def test_now_ok(capsys):
    in_path = pathlib.Path('tests', 'fixtures', 'basic', 'minimal-config.json')
    with pytest.raises(SystemExit) as exec_info:
        cli.now(conf=in_path)
        assert exec_info.value.code == 0
        out, err = capsys.readouterr()
        assert 'would extract html tables' in out.lower()
        assert not err


def test_translate_non_existing_html(capsys):
    in_path = pathlib.Path('does', 'not', 'exist', 'hypothetical.json')
    with pytest.raises(SystemExit) as exec_info:
        cli.now(conf=in_path)
        assert exec_info.value.code == 1
        out, err = capsys.readouterr()
        assert 'source' in out.lower()
        assert 'is no file' in out.lower()
        assert not err
